define((()=>(()=>{var e={758:(e,i,r)=>{r.p=document.querySelector("body").getAttribute("data-base-url")+"nbextensions/widget-bzvisualizer",window.require&&window.require.config({map:{"*":{"widget-bzvisualizer":"nbextensions/widget-bzvisualizer/index"}}}),e.exports={load_ipython_extension:function(){}}}},i={};function r(t){var n=i[t];if(void 0!==n)return n.exports;var o=i[t]={exports:{}};return e[t](o,o.exports,r),o.exports}return r.p="",r(758)})()));